import socket

class QQClient:
    def __init__(self):
        # hosts = ['tcpconn.tencent.com', 'tcpconn2.tencent.com', 'tcpconn3.tencent.com', 'tcpconn4.tencent.com']
        host = socket.gethostbyname("tcpconn.tencent.com")
        self._client = socket.socket()
        self._client.settimeout(7.0)
        self._client.connect((host, 443))

    def Send(self, body:bytes):
        length = len(body) + 2
        body = length.to_bytes(2, "big") + body
        self._client.send(body)
    
    def Recv(self)->bytes:
        return self._client.recv(10240)
    
    def Close(self):
        self._client.close()


import pcqq.utils as utils
tea = utils.Tea()

def Pack_0058(QQ):
    '''心跳包'''
    # tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()
    pack.SetBin(QQ.Utf8QQ)
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 58")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)
    pack.SetHex("03")

    body = pack.GetAll()
    return body


def Pack_0062(QQ) -> bytes:
    '''离线包'''
    # tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 62")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7 00 30 00 3A")
    pack.SetBin(tea.Encrypt(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00', QQ.SessionKey))
    pack.SetHex("03")

    body = pack.GetAll()
    return body


import pcqq.utils as utils
tea = utils.Tea()

def UnPack_0825(QQ, src: bytes)->bool:
    # tea = utils.Tea()
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:-1]
    dst = tea.Decrypt(dst,QQ.RandHead16)

    unpack.SetData(dst)
    Type = int(unpack.GetByte())
    unpack.GetShort()
    length = unpack.GetShort()
    QQ.PcToken0038From0825 = unpack.GetBin(length)
    unpack.GetBin(6)
    unpack.GetBin(4)
    QQ.LocalPcIp = unpack.GetBin(4)
    unpack.GetShort()

    if Type == 254: #需要重定向服务器
        unpack.GetBin(18)
        QQ.ConnectSeverIp = unpack.GetBin(4)
        QQ.ConnectSeverIpText = "重定向地址: %d.%d.%d.%d\n"%(
            QQ.ConnectSeverIp[0],
            QQ.ConnectSeverIp[1],
            QQ.ConnectSeverIp[2],
            QQ.ConnectSeverIp[3]
        )

        return True
    else:
        unpack.GetBin(6)
        QQ.ConnectSeverIp = unpack.GetBin(4)
        return False

def UnPack_0818(QQ, src: bytes)->tuple:
    '''解析二维码地址'''
    # tea = utils.Tea()
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:-1]
    dst = tea.Decrypt(dst,QQ.ShareKey)

    unpack.SetData(dst)
    unpack.GetBin(7)
    QQ.PcKeyFor0819 = unpack.GetBin(16)
    unpack.GetBin(4)
    length = int(unpack.GetShort())

    QQ.PcToken0038From0818 = unpack.GetBin(length)
    unpack.GetBin(4)
    length = unpack.GetShort()
    codeId = unpack.GetBin(length).decode()

    unpack.GetBin(4)
    length = unpack.GetShort()
    codeImg = unpack.GetBin(length)

    return (codeId,codeImg)

def UnPack_0819(QQ,src: bytes)->int:
    '''
    取二维码状态
    0 = 授权成功
    1 = 扫码成功
    2 = 未扫码
    3 = 空数据包
    :return: 状态码
    '''

    # tea = utils.Tea()
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:-1]
    dst = tea.Decrypt(dst,QQ.PcKeyFor0819)

    unpack.SetData(dst)
    stateId = unpack.GetByte()

    if stateId == 1:
        QQ.BinQQ = src[9:13]
        QQ.LongQQ = int.from_bytes(QQ.BinQQ,byteorder='big',signed=False)
        QQ.Utf8QQ = str(QQ.LongQQ).encode()

    if stateId == 0 and len(dst) > 1:
        unpack.GetShort()
        length = unpack.GetShort()
        QQ.PcToken0060From0819 = unpack.GetBin(int(length))
        unpack.GetShort()
        length = unpack.GetShort()
        QQ.PcKeyTgt = unpack.GetBin(int(length))
    return stateId

def UnPack_0836(QQ, src: bytes)->bool:
    # tea = utils.Tea()
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:-1]

    if len(dst) >= 100:
        dst = tea.Decrypt(tea.Decrypt(dst,QQ.ShareKey),QQ.PcKeyTgt)
    if len(dst) == 0:
        return False

    unpack.SetData(dst)
    Type = int(unpack.GetByte())

    if Type != 0:
        return False

    for i in range(5):
        tlvName = utils.Bin2HexTo(unpack.GetBin(2))
        tlvLength = int(unpack.GetShort())

        if tlvName == "1 9":
            unpack.GetShort()
            QQ.PcKeyFor0828Send = unpack.GetBin(16)

            length = int(unpack.GetShort())
            QQ.PcToken0038From0836 = unpack.GetBin(length)
            length = int(unpack.GetShort())
            unpack.GetBin(length)
            unpack.GetShort()
        elif tlvName == "1 7":
            unpack.GetBin(26)
            QQ.PcKeyFor0828Rev = unpack.GetBin(16)
            length = int(unpack.GetShort())
            QQ.PcToken0088From0836 = unpack.GetBin(length)
            length = tlvLength - 180
            unpack.GetBin(length)
        elif tlvName == "1 8":
            unpack.GetBin(8)
            length = int(unpack.GetByte())
            QQ.NickName = unpack.GetBin(length).decode()
        else:
            unpack.GetBin(tlvLength)

    if len(QQ.PcToken0088From0836) != 136 or len(QQ.PcKeyFor0828Send) != 16 or len(QQ.PcToken0038From0836) != 56:
        return False
    else:
        return True


def UnPack_0828(QQ, src: bytes):
    '''解析SessionKey'''
    # tea = utils.Tea()
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:-1]
    dst = tea.Decrypt(dst,QQ.PcKeyFor0828Rev)

    unpack.SetData(dst)
    unpack.GetBin(63)
    QQ.SessionKey = unpack.GetBin(16)

def UnPack_001D(QQ, src: bytes):
    '''解析Clientkey'''
    # tea = utils.Tea()
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(9)

    unpack.GetInt()
    unpack.GetBin(3)

    data = unpack.GetAll()
    data = data[0:-1]
    data = tea.Decrypt(data,QQ.SessionKey)

    unpack.SetData(data)
    unpack.GetBin(2)

    QQ.ClientKey = unpack.GetAll()

    if len(QQ.ClientKey) != 32:
        QQ.ClientKey = b''
        return


import pcqq.utils as utils
tea = utils.Tea()

def UnPack_00CE(QQ, src: bytes, msg)->bool:
    '''
    解析好友消息包
    :param src: 解密包体
    :param msg: Message类
    '''

    msg.MsgType = "friend"
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:len(dst) - 1]
    dst = tea.Decrypt(dst, QQ.SessionKey)
    if len(dst) == 0:
        return False
    msg.HeadBody = dst[0:16]
    unpack.SetData(dst)

    msg.FromQQ = unpack.GetInt()  # 触发者QQ
    msg.RecvQQ = unpack.GetInt()  # 接收者QQ
    unpack.GetBin(14)

    length = int(unpack.GetShort())
    unpack.GetBin(length)

    unpack.GetBin(2)  # QQ版本
    unpack.GetInt()  # 来源QQ
    unpack.GetInt()  # 自身QQ

    unpack.GetBin(16)  # 会话令牌
    unpack.GetBin(4)

    msg.MsgTime = unpack.GetInt()  # 接收时间

    try:
        unpack.GetBin(6)  # 头像、字体属性
        unpack.GetBin(5)  # 消息相关信息

        unpack.GetBin(24)
        length = int(unpack.GetShort())
        unpack.GetBin(length)  # 字体名称
        unpack.GetBin(2)

        msg.Parse(unpack.GetAll())
    except:
        return False
    return True


def UnPack_0017(QQ, src: bytes, msg):
    '''
    解析群消息包
    :param src: 解密包体
    :param msg: Message类
    '''
    msg.MsgType = "group"
    unpack = utils.PackDecrypt()

    unpack.SetData(src)
    unpack.GetBin(16)
    dst = unpack.GetAll()
    dst = dst[0:len(dst) - 1]
    dst = tea.Decrypt(dst, QQ.SessionKey)
    if dst == None or len(dst) == 0:
        return False

    msg.HeadBody = dst[0:16]
    unpack.SetData(dst)

    msg.FromGroup = unpack.GetInt()  # 接收群号
    msg.RecvQQ = unpack.GetInt()  # 自身QQ
    unpack.GetBin(10)
    typeOf = unpack.GetShort()  # 消息类型

    length = unpack.GetInt()
    unpack.GetBin(length)

    if len(unpack.GetAll()) < 5:
        return False
    unpack.GetInt()
    flag = unpack.GetByte()

    if typeOf == 82 and flag == 1:  # 群消息
        msg.FromQQ = unpack.GetInt()  # 接收QQ
        if msg.FromQQ == msg.RecvQQ:
            return False
        unpack.GetInt()  # 消息索引
        msg.MsgTime = unpack.GetInt()  # 接收时间
        unpack.GetBin(24)
        unpack.GetInt()  # 发送时间
        msg.MsgId = unpack.GetInt()  # 消息ID
        unpack.GetBin(8)

        length = int(unpack.GetShort())
        unpack.GetBin(length)  # 字体
        unpack.GetBin(2)

        # 以下是消息正文
        try:
            data = unpack.GetAll()
            pos = data.find(utils.Hex2Bin("4 0 C0 4 0 CA 4 0 F8 4 0"))
            unpack.SetData(data[pos + 35:])

            length = unpack.GetShort()
            msg.NickName = unpack.GetBin(length).decode()  # 发消息者昵称
            msg.Parse(data[0:pos])
        except Exception as err:
            return False
        return True


"""
Created on Mon Aug 25 at 18:02 2021

用前须知:
1. py-pcqq，这是一个本萌新使用Python编写的QQBot库，基于PC版QQ协议
2. 本项目仅作个人娱乐，功能极少，有更强大的功能需求请移步onebot
3. 本项目开源于Github，萌新代码写的很烂，各位大佬请多多包涵( 给点star吧，求求惹QAQ )
4. 本萌新的博客和Github项目首页中有关于该协议库的相关说明

联系方式:
@QQ: 2224825532
@Blog: http://blog.yeli.work
@Github: https://github.com/DawnNights

使用教程:
1. 使用pip install py-pcqq来安装这个协议库，并在代码中使用import pcqq来导入
2. 创建一个pcqq.QQBot类的实例，创建完毕后通过扫码登录获取机器人对象
3. 通过编写pcqq.Plugin类的子类，并重写match和handle方法来实现对机器人功能的编写
4. 调用RunBot方法使机器人开始运行

注意事项: 
1. 该协议库暂时只能使用扫码登录
2. 机器人能发送的消息包括(纯文本)
3. 机器人能接收的消息包括(at, 文本, 图片, 表情)
"""
from ._msg import Message
from ._bot import QQBot, Plugin

from ._client import QQClient
from ._struct import QQStruct



import random
import hashlib

def GetRandomBin(length: int)->bytes:
    '''生成指定长度的随机字节集'''
    dst = [random.randint(0,255).to_bytes(1,"big") for x in range(length)]
    return b''.join(dst)

def Bin2HexTo(src: bytes)->str:
    '''字节集转十六进制文本'''
    s = " ".join([hex(b)[2:].upper() for b in src])
    return s

def Hex2Bin(s: str)->bytes:
    '''十六进制文本转字节集'''
    dst = b''.join([int("0x"+i,16).to_bytes(1,"big") for i in s.split(" ")])
    return dst

def HashMD5(src: bytes)->bytes:
    '''将字节集编码为md5 16bit字节集'''
    s = ""
    dst = b''
    md = hashlib.md5(src)
    origin = md.hexdigest()
    for i in range(len(origin)):
        s += origin[i]
        if i % 2 != 0:
            int_hex = int(s, 16)
            dst += int_hex.to_bytes(1,"big")
            s = ""
    return dst

def GroupToGid(groupId:int)->int:
    '''群号转GID'''
    group = str(groupId)
    left = int(group[0:-6])
    if left >= 0 and left <= 10:
        right = group[-6:]
        gid = str(left + 202) + right
    elif left >= 11 and left <= 19:
        right = group[-6:]
        gid = str(left + 469) + right
    elif left >= 20 and left <= 66:
        left = int(str(left)[0:1])
        right = group[-7:]
        gid = str(left + 208) + right
    elif left >= 67 and left <= 156:
        right = group[-6:]
        gid = str(left + 1943) + right
    elif left >= 157 and left <= 209:
        left = int(str(left)[0:2])
        right = group[-7:]
        gid = str(left + 199) + right
    elif left >= 210 and left <= 309:
        left = int(str(left)[0:2])
        right = group[-7:]
        gid = str(left + 389) + right
    elif left >= 310 and left <= 335:
        left = int(str(left)[0:2])
        right = group[-7:]
        gid = str(left + 349) + right
    elif left >= 336 and left <= 386:
        left = int(str(left)[0:3])
        right = group[-6:]
        gid = str(left + 2265) + right
    elif left >= 387 and left <= 499:
        left = int(str(left)[0:3])
        right = group[-6:]
        gid = str(left + 3490) + right
    elif left >= 500:
        return int(group)
    return int(gid)
    # i.to_bytes(4,'big')


import pcqq.utils as utils

class QQStruct:
    def __init__(self):
        self.BinQQ = b''
        self.LongQQ = 0
        self.Utf8QQ = b''
        self.Redirection = 0

        self.Time = b''
        self.NickName = ''

        self.RandHead16 = b''
        self.SessionKey = b''
        self.ClientKey = b''
        self.ShareKey = b''
        self.PublicKey = b''

        self.PcKeyFor0819 = b''
        self.PcKeyTgt = b''
        self.PcKeyFor0828Send = b''
        self.PcKeyFor0828Rev = b''
        self.PcToken0038From0825 = b''
        self.PcToken0038From0818 = b''
        self.PcToken0060From0819 = b''
        self.PcToken0038From0836 = b''
        self.PcToken0088From0836 = b''

        self.LocalPcIp = b''
        self.ConnectSeverIp = b''
        self.ConnectSeverIpText = ""


import pcqq.utils as utils
tea = utils.Tea()

def Pack_0825(QQ, IsLogin:bool=False)->bytes:
    body = b''
    # tea = utils.Tea()
    pack = utils.PackEncrypt()
    QQ.RandHead16 = utils.GetRandomBin(16)

    if not IsLogin:
        pack.SetHex(
            "00 18 00 16 00 01 00 00 04 4C 00 00 00 01 00 00 15 51 00 00 00 00 00 00 00 00 00 04 00 0C 00 00 00 08 71 72 5F 6C 6F 67 69 6E 03 09 00 08 00 01 00 00 00 00 00 04 01 14 00 1D")
        pack.SetHex("01 02")
        pack.SetShort(len((QQ.PublicKey)))
        pack.SetBin(QQ.PublicKey)
        body = tea.Encrypt(pack.GetAll(), QQ.RandHead16)

        pack.Empty()
        pack.SetHex("02 36 39")
        pack.SetHex("08 25")
        pack.SetBin(utils.GetRandomBin(2))
        pack.SetHex("00 00 00 00 03 00 00 00 01 01 01 00 00 67 B7 00 00 00 00")
        pack.SetBin(QQ.RandHead16)
        pack.SetBin(body)
        pack.SetHex("03")
        body = pack.GetAll()
    else:
        pack.SetHex("00 18 00 16 00 01 00 00 04 4C 00 00 00 01 00 00 15 51")
        pack.SetBin(QQ.BinQQ)
        pack.SetHex("00 00 00 00 03 09 00 08 00 01")
        pack.SetBin(QQ.ConnectSeverIp)
        pack.SetHex("00 01 00 36 00 12 00 02 00 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00 01 14 00 1D")
        pack.SetHex("01 02")
        pack.SetShort(len(QQ.PublicKey))
        pack.SetBin(QQ.PublicKey)
        body = tea.Encrypt(pack.GetAll(), QQ.RandHead16)

        pack.Empty()
        pack.SetHex("02 36 39")
        pack.SetHex("08 25")
        pack.SetBin(utils.GetRandomBin(2))
        pack.SetBin(QQ.BinQQ)
        pack.SetHex("03 00 00 00 01 01 01 00 00 67 B7 00 00 00 00")
        pack.SetBin(QQ.RandHead16)
        pack.SetBin(body)
        pack.SetHex("03")
        body = pack.GetAll()
    return body


def Pack_0818(QQ) -> bytes:
    '''获取二维码'''
    # tea = utils.Tea()
    pack = utils.PackEncrypt()
    QQ.RandHead16 = utils.GetRandomBin(16)

    pack.Empty()
    pack.SetHex("00 19 00 10 00 01 00 00 04 4C 00 00 00 01 00 00 15 51 00 00 01 14 00 1D")
    pack.SetHex("01 02")
    pack.SetShort(len(QQ.PublicKey))
    pack.SetBin(QQ.PublicKey)
    pack.SetHex("03 05 00 1E 00 00 00 00 00 00 00 05 00 00 00 04 00 00 00 00 00 00 00 48 00 00 00 02 00 00 00 02 00 00")
    body = tea.Encrypt(pack.GetAll(), QQ.RandHead16)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("08 18")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetHex("00 00 00 00 03 00 00 00 01 01 01 00 00 67 B7 00 00 00 00")
    pack.SetBin(QQ.RandHead16)
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()

    return body


def Pack_0819(QQ, codeId: str, IsLogin: bool = False) -> bytes:
    '''
    检查二维码状态
    :param codeId: 二维码ID
    :param IsLogin: 授权登录(True)/取二维码验证状态(False)
    '''

    # tea = utils.Tea()
    pack = utils.PackEncrypt()
    QQ.RandHead16 = utils.GetRandomBin(16)

    pack.SetHex("00 19 00 10 00 01 00 00 04 4C 00 00 00 01 00 00 15 51 00 00 03 01 00 22")

    pack.SetShort(len(codeId))
    pack.SetStr(codeId)
    if IsLogin:
        pack.SetHex("03 14 00 02 00 00")
    body = tea.Encrypt(pack.GetAll(), QQ.PcKeyFor0819)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("08 19")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetHex("00 00 00 00 03 00 00 00 01 01 01 00 00 67 B7 00 00 00 00 00 30 00 3A")
    pack.SetShort(len(QQ.PcToken0038From0818))
    pack.SetBin(QQ.PcToken0038From0818)
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()

    return body


def Pack_0836(QQ) -> bytes:
    tlv = utils.Tlv()
    # tea = utils.Tea()
    pack = utils.PackEncrypt()
    QQ.RandHead16 = utils.GetRandomBin(16)

    pack.Empty()
    pack.SetBin(tlv.Tlv112(QQ.PcToken0038From0825))
    pack.SetBin(tlv.Tlv30F("DawnNights-PCQQ"))
    pack.SetBin(tlv.Tlv005(QQ.BinQQ))
    pack.SetBin(tlv.Tlv303(QQ.PcToken0060From0819))
    pack.SetBin(tlv.Tlv015())
    pack.SetBin(tlv.Tlv01A(QQ.PcKeyTgt))

    pack.SetBin(tlv.Tlv018(QQ.BinQQ, QQ.Redirection))
    pack.SetBin(tlv.Tlv103())

    pack.SetBin(tlv.Tlv312())
    pack.SetBin(tlv.Tlv508())
    pack.SetBin(tlv.Tlv313())
    pack.SetBin(tlv.Tlv102(QQ.PcToken0038From0825))
    body = tea.Encrypt(pack.GetAll(), QQ.ShareKey)

    pack.Empty()

    pack.SetHex("02 36 39")
    pack.SetHex("08 36")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)

    pack.SetHex("03 00 00 00 01 01 01 00 00 67 B7 00 00 00 00 00 01")
    pack.SetHex("01 02")
    pack.SetShort(len(QQ.PublicKey))
    pack.SetBin(QQ.PublicKey)
    pack.SetHex("00 00 00 10")
    pack.SetBin(QQ.RandHead16)
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_0828(QQ) -> bytes:
    tlv = utils.Tlv()
    # tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()

    pack.SetBin(tlv.Tlv007(QQ.PcToken0088From0836))
    pack.SetBin(tlv.Tlv00C(QQ.ConnectSeverIp))
    pack.SetBin(tlv.Tlv015())
    pack.SetBin(tlv.Tlv036())
    pack.SetBin(tlv.Tlv018(QQ.BinQQ, QQ.Redirection))
    pack.SetBin(tlv.Tlv01F())
    pack.SetBin(tlv.Tlv105())
    pack.SetBin(tlv.Tlv10B())
    pack.SetBin(tlv.Tlv02D())
    body = tea.Encrypt(pack.GetAll(), QQ.PcKeyFor0828Send)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("08 28")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7 00 30 00 3A")
    pack.SetShort(len(QQ.PcToken0038From0836))
    pack.SetBin(QQ.PcToken0038From0836)
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_001D(QQ) -> bytes:
    '''更新Clientkey'''
    # tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()

    pack.SetHex("11")
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 1D")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_00EC(QQ, state: int) -> bytes:
    '''
    置上线状态
    :param state:
    1 = 在线
    2 = Q我吧
    3 = 离开
    4 = 忙碌
    5 = 请勿打扰
    6 = 隐身
    '''
    # tea = utils.Tea()
    pack = utils.PackEncrypt()
    pack.SetHex("01 00")

    if state == 1:
        pack.SetHex("0A")
    elif state == 2:
        pack.SetHex("3C")
    elif state == 3:
        pack.SetHex("1E")
    elif state == 4:
        pack.SetHex("32")
    elif state == 5:
        pack.SetHex("46")
    elif state == 6:
        pack.SetHex("28")
    else:
        pack.SetHex("0A")

    pack.SetHex("00 01 00 01 00 04 00 00 00 00")
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)
    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 EC")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


import re
import os
import time
import asyncio
import platform
import threading
import logging as log
from pcqq.package import *
from ._msg import *
from ._client import QQClient
from ._struct import QQStruct

log.basicConfig(level=log.DEBUG, format="[pcqq] time=\"%(asctime)s\" level=%(levelname)s msg=\"%(message)s\"")


class Plugin:
    def __init__(self, bot, msg):
        self.Bot = bot
        self.Msg = msg
        self.Args = []

    def match(self):
        '''判断接收消息是否被匹配'''
        return self.on_full_match("hello")
    
    def handle(self):
        '''当消息匹配时所触发的事件'''
        self.send_msg("hello world")

    def on_full_match(self, keyword:str="") -> bool:
        '''
        完全匹配模式->当接收消息为keyword时满足匹配
        :param keyword: 匹配关键词
        :return: 是否匹配
        '''
        return self.Msg.MsgText == keyword

    def on_reg_match(self, pattern:str="") -> bool:
        '''
        正则匹配模式->当该正则解析接收消息结果不为空时完成匹配，解析结果存至Args属性中
        :param pattern: 匹配表达式
        :return: 是否匹配
        '''
        self.Args = re.findall(pattern, self.Msg.MsgText)
        if self.Args != []:
            return True
        return False
    
    def is_at_me(self) -> bool:
        '''
        判断机器人是否被艾特
        '''
        if f"[PQ:at,qq={self.Bot.LongQQ}]" in self.Msg.MsgText:
            self.Msg.MsgText = self.Msg.MsgText.replace(f"[PQ:at,qq={self.Bot.LongQQ}]","")
            return True
        return False
    
    def is_admin_user(self) -> bool:
        '''
        判断消息发送者是否为机器人的主人
        '''
        if self.Msg.FromQQ in self.Bot.adminUsers:
            return True
        return False

    def on_common_match(self, keyword:str, promat:str="") -> bool:
        '''
        命令匹配模式->当接收消息的开头为keyword时满足匹配，并将参数存至Args属性中；若参数为空且promat不为空字符串，则向发送者索取参数
        :param keyword: 匹配关键词
        :param promat: 参数索取语句
        :return: 是否匹配
        '''
        if self.Msg.MsgText.find(keyword) == 0:
            arg = self.Msg.MsgText[len(keyword):].strip()
            if arg == "":
                if promat != "":
                    self.send_msg(promat)
                    self.Bot._table[self.Msg.FromQQ] = keyword
                return False
            self.Args.append(arg)
            return True
        return False

    def send_msg(self, msgText: str):
        if self.Msg.MsgType == "friend":
            self.Bot.SendUserMsg(self.Msg.FromQQ, msgText)
        elif self.Msg.MsgType == "group":
            self.Bot.SendGroupMsg(self.Msg.FromGroup, msgText)

class QQBot:
    def __init__(self, adminUsers:list=[]):
        '''
        :param adminUser: 机器人管理者的QQ
        '''
        self.adminUsers = adminUsers

        self.QQ = QQStruct()
        self.QQ.RandHead16 = utils.GetRandomBin(16)
        self.QQ.ShareKey = utils.Hex2Bin("FD 0B 79 78 31 E6 88 54 FC FA EA 84 52 9C 7D 0B")
        self.QQ.PublicKey = utils.Hex2Bin("03 94 3D CB E9 12 38 61 EC F7 AD BD E3 36 91 91 07 01 50 BE 50 39 1C D3 32")

        self.Client = QQClient()
        self._GetQrCode()

        log.info(f"账号{self.QQ.LongQQ}登录成功")
        log.info(f"欢迎登录，尊敬的用户【{self.QQ.NickName}】")

        h = threading.Thread(target=self._Heart)
        h.start()

    def ListenMsg(self)->Message:
        '''监听一条消息'''
        msg = Message()
        src = self.Client.Recv()

        if src[5:7] == USERMESSAGE:  # 好友消息
            if UnPack_00CE(self.QQ, src, msg):
                log.info(f"收到用户({msg.FromQQ})消息: {msg.MsgText}")
                self.Client.Send(Pack_00CE(self.QQ, msg.HeadBody, src[7:9]))
                self.Client.Send(Pack_0319(self.QQ, msg))
        elif src[5:7] == GROUPMESSAGE:  # 群消息
            if UnPack_0017(self.QQ, src, msg):
                log.info(f"收到群({msg.FromGroup})消息 {msg.NickName}: {msg.MsgText}")
                self.Client.Send(Pack_0017(self.QQ, msg.HeadBody, src[7:9]))
                self.Client.Send(Pack_0002_receipt(self.QQ, msg))
        
        return msg

    def CallPlugin(self, msg:Message):
        '''执行编写的插件功能'''
        for plugin in Plugin.__subclasses__():
            demo = plugin(self, msg)
            if demo.match():
                demo.handle()
    
    def RunBot(self):
        '''开始运行机器人'''
        self._table = {}
        loop = asyncio.get_event_loop()

        async def repeat(bot:QQBot):
            msg = bot.ListenMsg()
            if msg.FromQQ in bot._table.keys():
                msg.MsgText = bot._table[msg.FromQQ] + msg.MsgText
                bot._table.pop(msg.FromQQ)
            bot.CallPlugin(msg)

        while True:
            try:
                loop.run_until_complete(repeat(self))
            except:
                pass    # 忽略所有异常使程序正常运行
    
    def SendGroupMsg(self, groupID:int, msgText:str):
        '''
        发送群消息
        :param groupID: 群号
        :param msgText: 发送文本
        '''
        self.Client.Send(Pack_0002(self.QQ, groupID, msgText))
        if self.Client.Recv() != b"":
            log.info(f'{self.QQ.NickName}: "{msgText}" -> GroupID: {groupID}')
        else:
            log.warning(f"向群{groupID}发送消息失败")

    def SendUserMsg(self, userID:int, msgText:str):
        '''
        发送用户消息
        :param userID: 用户账号
        :param msgText: 发送文本
        '''
        self.Client.Send(Pack_00CD(self.QQ, userID, msgText))
        if self.Client.Recv() != b"":
            log.info(f'{self.QQ.NickName}: "{msgText}" -> UserID: {userID}')
        else:
            log.warning(f"向用户{userID}发送消息失败")

    def _Heart(self):
        '''每隔60秒发送一次心跳包'''
        self.Client.Send(Pack_0058(self.QQ))
        h = threading.Timer(60.0, self._Heart)
        h.start()

    def _GetQrCode(self):
        '''获取登录二维码并等待扫码'''

        # 获取登录二维码
        self.Client.Send(Pack_0825(self.QQ))
        UnPack_0825(self.QQ, self.Client.Recv())

        # 解析登录二维码
        self.Client.Send(Pack_0818(self.QQ))
        QrCodeID, QrCodeImg = UnPack_0818(self.QQ, self.Client.Recv())

        while True:
            try:
                with open("QrCode.jpg", "wb") as f:
                    f.write(QrCodeImg)
                break
            except:
                continue

        if platform.system() == "Windows":
            log.info("登录二维码获取成功，已保存至"+os.getcwd()+"\\QrCode.jpg")
            os.startfile("QrCode.jpg")
        else:
            log.info("登录二维码获取成功，已保存至"+os.getcwd()+"/QrCode.jpg")

        # 等待扫码登录
        stateID = -1
        for x in range(60):
            self.Client.Send(Pack_0819(self.QQ, QrCodeID))
            stateID = UnPack_0819(self.QQ, self.Client.Recv())
            log.debug("二维码状态: " + {0: "授权成功", 1: "扫码成功", 2: "未扫码", 3: "空数据包"}.get(stateID))
            if stateID == 0:
                self.Client.Send(Pack_0825(self.QQ, True))
                UnPack_0825(self.QQ, self.Client.Recv())
                log.debug(f"地址重定向为{self.QQ.ConnectSeverIpText}")
                try:
                    self.Client.Send(Pack_0836(self.QQ))
                    UnPack_0836(self.QQ, self.Client.Recv())
                except:
                    input("登录失败，可能是您的设备开启了登录保护\n请在手机QQ的设置-账号安全-登录设备管理中关闭[登录保护]\n")
                    exit()

                self.Client.Send(Pack_0828(self.QQ))
                UnPack_0828(self.QQ, self.Client.Recv())

                self.Client.Send(Pack_00EC(self.QQ, 1))  # 置登录状态为上线

                self.Client.Send(Pack_001D(self.QQ))
                UnPack_001D(self.QQ, self.Client.Recv())
                break
            elif stateID == 1:
                log.info(f"当前扫码账号为: {self.QQ.LongQQ}，请确认登录")
            time.sleep(1)

        if stateID != 0:
            input("扫码登录超时，请重新运行程序！\n")
            exit()



from ._util import Hex2Bin

class PackEncrypt():
    '''打包'''
    def __init__(self):
        self._src = b''

    def Empty(self):
        '''清空'''
        self._src = b''

    def SetBin(self, data: bytes):
        '''置字节集'''
        self._src = self._src + data

    def SetHex(self, s: str):
        '''置文本十六进制格式'''
        self._src = self._src + Hex2Bin(s)

    def SetShort(self, num: int):
        '''置短整数'''
        self._src = self._src + num.to_bytes(2,"big")

    def SetInt(self, num: int):
        '''置整数'''
        self._src = self._src + num.to_bytes(4,"big")

    def SetVarInt(self, num: int):
        '''置变长整数'''
        n = 0
        buf = [None for x in range(5)]
        while num > 127:
            buf[n] = 0x80 | num&0x7F
            num = num >> 7
            n += 1
        buf[n] = num
        self._src = self._src + b''.join([x.to_bytes(1,"big") for x in buf])

    def SetStr(self,s: str):
        '''置字符串'''
        self._src = self._src + s.encode()

    def GetAll(self,command="") -> bytes:
        '''取所有数据'''
        if command != "":
            pass
        return self._src


import time
import pcqq.utils as utils

def Pack_0002(QQ, groupId: int, content: str) -> bytes:
    '''
    构建发送群文本消息包
    :param groupId: 发送群号
    :param content: 消息内容
    '''

    tea = utils.Tea()
    pack = utils.PackEncrypt()
    QQ.Time = int(time.time()).to_bytes(4, "big")
    Msg = content.encode()

    pack.Empty()
    pack.SetHex("00 01 01 00 00 00 00 00 00 00 4D 53 47 00 00 00 00 00")
    pack.SetBin(QQ.Time)
    pack.SetBin(QQ.Time[::-1])
    pack.SetHex("00 00 00 00 09 00 86 00 00 06 E5 AE 8B E4 BD 93 00 00 01")
    pack.SetShort(len(Msg) + 3)
    pack.SetHex("01")
    pack.SetShort(len(Msg))
    pack.SetBin(Msg)
    body = pack.GetAll()

    pack.Empty()
    pack.SetHex("2A")
    pack.SetInt(groupId)
    pack.SetShort(len(body))
    pack.SetBin(body)
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 02")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_00CD(QQ, userId: int, content: str) -> bytes:
    '''
    构建发送好友文本消息包
    :param userId: 发送帐号
    :param content: 消息内容
    '''

    tea = utils.Tea()
    pack = utils.PackEncrypt()
    QQ.Time = int(time.time()).to_bytes(4, "big")
    Msg = content.encode()

    pack.Empty()
    pack.SetBin(QQ.BinQQ)
    pack.SetInt(userId)
    pack.SetHex("00 00 00 08 00 01 00 04 00 00 00 00 36 39")
    pack.SetBin(QQ.BinQQ)
    pack.SetInt(userId)

    pack.SetBin(utils.HashMD5(QQ.Time))

    pack.SetHex("00 0B 4A B6")
    pack.SetBin(QQ.Time)
    pack.SetHex("02 55 00 00 00 00 01 00 00 00 01 4D 53 47 00 00 00 00 00")
    pack.SetBin(QQ.Time)
    pack.SetBin(QQ.Time[::-1])
    pack.SetHex("00 00 00 00 09 00 86 00 00 06 E5 AE 8B E4 BD 93 00 00 01")
    pack.SetShort(len(Msg) + 3)
    pack.SetHex("01")
    pack.SetShort(len(Msg))
    pack.SetBin(Msg)
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 CD")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_0017(QQ, sendBody: bytes, sequence: bytes) -> bytes:
    '''
    确认群消息已读
    :param sendBody: 消息包解密前16位
    '''
    tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()
    pack.SetBin(sendBody)
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 17")
    pack.SetBin(sequence)
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)

    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_00CE(QQ, sendBody: bytes, sequence: bytes) -> bytes:
    '''
    确认好友消息已读
    :param sendBody: 消息包解密前16位
    '''
    tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()
    pack.SetBin(sendBody)
    body = tea.Encrypt(sendBody, QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 CE")
    pack.SetBin(sequence)
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)

    pack.SetHex("03")
    body = pack.GetAll()
    return body


def Pack_0002_receipt(QQ, msg) -> bytes:
    '''群消息回执包'''
    tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()
    pack.SetHex("29")
    pack.SetBin(utils.GroupToGid(msg.FromGroup).to_bytes(4, "big"))
    pack.SetHex("02")
    pack.SetBin(msg.MsgId.to_bytes(4, "big"))
    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("00 02")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("02 00 00 00 01 01 01 00 00 67 B7")
    pack.SetBin(body)
    pack.SetHex("03")

    body = pack.GetAll()
    return body


def Pack_0319(QQ, msg) -> bytes:
    '''好友消息回执包'''
    tea = utils.Tea()
    pack = utils.PackEncrypt()

    pack.Empty()
    pack.SetHex("08 01")
    pack.SetHex("12 03 98 01 00")

    pack.SetHex("0A 0E 08")
    pack.SetVarInt(msg.FromQQ)
    pack.SetHex("10")
    pack.SetVarInt(msg.MsgTime)
    pack.SetHex("20 00")
    body = pack.GetAll()

    pack.Empty()
    pack.SetHex("00 00 00 07")
    pack.SetInt(len(body) - 7)
    pack.SetBin(body)

    body = tea.Encrypt(pack.GetAll(), QQ.SessionKey)

    pack.Empty()
    pack.SetHex("02 36 39")
    pack.SetHex("03 19")
    pack.SetBin(utils.GetRandomBin(2))
    pack.SetBin(QQ.BinQQ)
    pack.SetHex("04 00 00 00 01 01 01 00 00 6A 0F 00 00 00 00 00 00 00 00")

    pack.SetBin(body)
    pack.SetHex("03")
    body = pack.GetAll()
    return body


import struct

class Tea():
    def xor(self, a, b):
        op = 0xffffffff
        a1, a2 = struct.unpack(b'>LL', a[0:8])
        b1, b2 = struct.unpack(b'>LL', b[0:8])
        return struct.pack(b'>LL', (a1 ^ b1) & op, (a2 ^ b2) & op)

    def code(self, v, k):
        n = 16
        op = 0xffffffff
        delta = 0x9e3779b9
        k = struct.unpack(b'>LLLL', k[0:16])
        y, z = struct.unpack(b'>LL', v[0:8])
        s = 0
        for i in range(n):
            s += delta
            y += (op & (z << 4)) + k[0] ^ z + s ^ (op & (z >> 5)) + k[1]
            y &= op
            z += (op & (y << 4)) + k[2] ^ y + s ^ (op & (y >> 5)) + k[3]
            z &= op
        r = struct.pack(b'>LL', y, z)
        return r

    def decipher(self, v, k):
        n = 16
        op = 0xffffffff
        y, z = struct.unpack(b'>LL', v[0:8])
        a, b, c, d = struct.unpack(b'>LLLL', k[0:16])
        delta = 0x9E3779B9
        s = (delta << 4) & op
        for i in range(n):
            z -= ((y << 4) + c) ^ (y + s) ^ ((y >> 5) + d)
            z &= op
            y -= ((z << 4) + a) ^ (z + s) ^ ((z >> 5) + b)
            y &= op
            s -= delta
            s &= op
        return struct.pack(b'>LL', y, z)

    def Encrypt(self, v, k):
        END_CHAR = b'\0'
        FILL_N_OR = 0xF8
        vl = len(v)
        filln = (8 - (vl + 2)) % 8 + 2
        fills = b''
        for i in range(filln):
            fills = fills + bytes([220])
        v = (bytes([(filln - 2) | FILL_N_OR])
             + fills
             + v
             + END_CHAR * 7)
        tr = b'\0' * 8
        to = b'\0' * 8
        r = b''
        o = b'\0' * 8
        for i in range(0, len(v), 8):
            o = self.xor(v[i:i + 8], tr)
            tr = self.xor(self.code(o, k), to)
            to = o
            r += tr
        return r

    def Decrypt(self, v, k):
        l = len(v)
        prePlain = self.decipher(v, k)
        pos = (prePlain[0] & 0x07) + 2
        r = prePlain
        preCrypt = v[0:8]
        for i in range(8, l, 8):
            x = self.xor(self.decipher(self.xor(v[i:i + 8], prePlain), k), preCrypt)
            prePlain = self.xor(x, preCrypt)
            preCrypt = v[i:i + 8]
            r += x
        if r[-7:] != b'\0' * 7:
            return None
        return r[pos + 1:-7]





#!/usr/bin/env python
# coding: utf-8

from setuptools import setup,find_packages

setup(
    name='py-pcqq',
    version='0.2.4',
    author='DawnNights',
    author_email='2224825532@qq.com',
    url='https://github.com/DawnNights/py-pcqq',
    description=u'一个使用pcqq协议的简易python qqbot库',
    long_description = open("README.rst", encoding='utf-8').read(),
    # long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[],
    entry_points={},
    data_files=["README.md"]
)



from ._tea import Tea
from ._util import GetRandomBin
from ._pack import PackEncrypt


class Tlv():
    def _TlvPack(self,TlvCmd: str, TlvBin: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex(TlvCmd)
        pack.SetShort(len(TlvBin))
        pack.SetBin(TlvBin)
        data = pack.GetAll()
        return data

    def Tlv112(self,Token0038From0825: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetBin(Token0038From0825)
        return self._TlvPack("01 12", pack.GetAll())

    def Tlv30F(self,PcName: str) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetShort(len(PcName))
        pack.SetStr(PcName)
        return self._TlvPack("03 0F", pack.GetAll())

    def Tlv005(self,BinQQ: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 02")
        pack.SetBin(BinQQ)
        return self._TlvPack("00 05", pack.GetAll())

    def Tlv303(self,PcToken0060From0819: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetBin(PcToken0060From0819)
        return self._TlvPack("03 03", pack.GetAll())

    def Tlv015(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01 01 74 83 F2 C3 00 10 14 FE 77 FC 00 00 00 00 00 00 00 00 00 00 00 00 02 17 65 6E 9D 00 10 78 8A 33 DD 00 76 A1 78 EB 8E 5B BB FF 17 D0 10")
        return self._TlvPack("00 15", pack.GetAll())

    def Tlv01A(self,Tgtkey: bytes) -> bytes:
        tea = Tea()
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01 01 74 83 F2 C3 00 10 14 FE 77 FC 00 00 00 00 00 00 00 00 00 00 00 00 02 17 65 6E 9D 00 10 78 8A 33 DD 00 76 A1 78 EB 8E 5B BB FF 17 D0 10")
        return self._TlvPack("00 1A", tea.Encrypt(pack.GetAll(),Tgtkey))

    def Tlv018(self, BinQQ: bytes, Redirect: int) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01 00 00 04 4C 00 00 00 01 00 00 15 51")
        pack.SetBin(BinQQ)
        pack.SetBin(Redirect.to_bytes(2,"big"))
        pack.SetHex("00 00")
        return self._TlvPack("00 18", pack.GetAll())

    def Tlv103(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01 00 10")
        pack.SetBin(GetRandomBin(16))
        return self._TlvPack("01 03", pack.GetAll())

    def Tlv312(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("01 00 00 00 00")
        return (self._TlvPack("03 12", pack.GetAll()))

    def Tlv313(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("01 01 02 00 10 EE 47 7F A4 BC D6 EE 65 02 65 4D E9 43 38 4C 3D 00 00 00 EB")
        return self._TlvPack("03 13", pack.GetAll())

    def Tlv102(self,Token0038From0825: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01")
        pack.SetBin(GetRandomBin(16))
        pack.SetShort(len(Token0038From0825))
        pack.SetBin(Token0038From0825)
        pack.SetHex("00 14")
        pack.SetBin(GetRandomBin(20))
        return self._TlvPack("01 02", pack.GetAll())

    def Tlv007(self,PcToken0088From0836: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetBin(PcToken0088From0836)
        return self._TlvPack("00 07", pack.GetAll())

    def Tlv00C(self,ServerIp: bytes) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 02 00 01 00 00 00 00 00 00 00 00")
        pack.SetBin(ServerIp)
        pack.SetHex("00 50 00 00 00 00")
        return self._TlvPack("00 0C", pack.GetAll())

    def Tlv036(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 02 00 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00")
        return self._TlvPack("00 36", pack.GetAll())

    def Tlv01F(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01")
        pack.SetBin(GetRandomBin(32))
        return self._TlvPack("00 1F", pack.GetAll())

    def Tlv105(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01 01 02 00 14 01 01 00 10")
        pack.SetBin(GetRandomBin(16))
        pack.SetHex("00 14 01 02 00 10")
        pack.SetBin(GetRandomBin(16))
        return self._TlvPack("01 05", pack.GetAll())

    def Tlv10B(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 02")
        pack.SetBin(GetRandomBin(17))
        pack.SetHex("10 00 00 00 00 00 00 00 02 00 63 3E 00 63 02 04 00 03 07 00 04 00 49 F5 00 00 00 00 78 8A 33 DD 00 76 A1 78 EB 8E 5B BB FF 17 D0 10 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 FE 26 81 75 EC 2A 34 EF 02 3E 50 39 6D B1 AF CC 9F EA 54 E1 70 CC 6C 9E 4E 63 8B 51 EC 7C 84 5C 68 00 00 00 00")
        return self._TlvPack("01 0B", pack.GetAll())

    def Tlv02D(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("00 01")
        pack.SetHex("C0 A8 74 83")
        return self._TlvPack("00 2D", pack.GetAll())

    def Tlv508(self) -> bytes:
        pack = PackEncrypt()
        pack.Empty()
        pack.SetHex("01 00 00 00 02")
        return self._TlvPack("05 08", pack.GetAll())





class PackDecrypt():
    '''解包'''
    def __init__(self):
        self._src = b''
        self._position = 0

    def SetData(self,src: bytes):
        '''置字节集'''
        self._src = src

    def GetBin(self,length: int)->bytes:
        '''取指定长度字节集'''
        temp = self._src[0:length]
        self._src = self._src[length:]
        self._position += length
        return temp

    def GetShort(self)->int:
        '''取短整数'''
        n = int.from_bytes(self._src[0:2],byteorder='big',signed=False)
        self._src = self._src[2:]
        self._position += 2
        return n

    def GetInt(self)->int:
        '''取整数'''
        n = int.from_bytes(self._src[0:4],byteorder='big',signed=False)
        self._src = self._src[4:]
        self._position += 4
        return n

    def GetByte(self)->int:
        '''取字节'''
        b = self._src[0]
        self._src = self._src[1:]
        self._position += 1
        return b

    def GetAll(self)->bytes:
        '''取所有字节集'''
        return self._src

    def GetPosition(self)->int:
        '''取position'''
        return self._position